(* KGL Semantically Checked ASt *)


open Ast

(* Expressions *)
type s_expr = 
    S_IntLit of int
  | S_FloatLit of float
  | S_BoolLit of bool
  | S_CharLit of char
  | S_StrLit of string
  | S_Id of string * var_type 
  | S_Assign of s_expr * asnop * s_expr * var_type   
  | S_Not of s_expr * var_type 
  | S_Binop of s_expr * op * s_expr * var_type 
  | S_In of s_expr * s_expr * var_type * var_type
  | S_Call of string * s_expr list * var_type
  | S_Value of s_expr * s_expr  * var_type * var_type 
  | S_ListLit of s_expr list * var_type
  | S_DictLit of (s_expr * s_expr) list  * var_type * var_type  
  | S_GraphLit of (s_expr * s_expr * s_expr) list
  | S_Null
  | S_Noexpr 

(* Variable declaration *)
type s_var_decl =  {
  s_vtype: var_type;
  s_vname: string;
  s_vinit: s_expr; 
}

(* Statement *)
and s_stmt = 
    S_Block of symbol_table * s_stmt list
  | S_Variable of s_var_decl
  | S_Expr of s_expr * var_type 
  | S_Return of s_expr
  | S_If of s_expr * s_stmt * s_stmt 
  | S_For of s_expr * s_expr * s_expr * s_stmt
  | S_Foreach of s_expr * s_expr * var_type * s_stmt
  | S_While of s_expr * s_stmt
  | S_Continue
  | S_Break

(* Function declaration *)
and s_func_decl = {
  s_fname: string;
  s_formals: (var_type * string) list;
  s_rtype: var_type;
  s_body: s_stmt list;
}

(* Program *)
and s_program = {
  s_var_decls: s_var_decl list;
  s_func_decls: s_func_decl list;
  s_symbols : symbol_table;
}

(* Symbol table *)
and symbol_table = {
  parent: symbol_table option;  (* parent symbol table *)
  mutable variables: (string * var_type) list;
}

(* Environment for each function /statment block *)
type environment = {
  scope: symbol_table;         
  mutable funcs: (var_type * string * var_type list) list; 
  return_type: var_type;      
  in_loop: bool;                (* if in loop, to check break/continue *) 
}







